/*************** schema format from the backend *************** */

/** Schema type, consist of nodes and edges */
export type SchemaFromBackend = {
  edges: SchemaEdge[];
  nodes: SchemaNode[];
};

export type SchemaStatsFromBackend = {
  edgeStats: {
    type: string;
    count: number;
  }[];
  nodeStats: {
    key: string;
    labels: string[];
    count: number;
  }[];
};

export type SchemaAttributeTypes =
  | 'string'
  | 'float'
  | 'int'
  | 'bool'
  | 'boolean'
  | 'date'
  | 'time'
  | 'datetime'
  | 'duration'
  | 'number'
  | 'location';

export type DimensionType = 'categorical' | 'numerical' | 'temporal' | 'spatial';

/** Attribute type, consist of a name */
export type SchemaAttribute = {
  name: string;
  type: SchemaAttributeTypes;
};

/** Node type, consist of a name and a list of attributes */
export type SchemaNode = {
  name: string;
  attributes: SchemaAttribute[];
  type?: string;
};

export type SchemaGraphInference = {
  nodes: Record<string, Record<string, DimensionType>>; // node key -> attribute label -> dimension
  edges: Record<string, Record<string, DimensionType>>;
};

export type SchemaGraphStats = {
  nodes: {
    count: number;
    stats: Record<string, NodeOrEdgeStats>; // node key -> Stats
  };
  edges: {
    count: number;
    stats: Record<string, NodeOrEdgeStats>; // edge key -> Stats
  };
};

export type NodeOrEdgeStats = {
  key: string;
  type?: string;
  count: number;
  attributeStats: Array<SchemaAttributeStats>;
};

export type SchemaAttributeStats =
  | BooleanSchemaAttributeStats
  | StringSchemaAttributeStats
  | NumberSchemaAttributeStats
  | DateTimeSchemaAttributeStats
  | LocationSchemaAttributeStats;

export type CommonSchemaAttributeStats = {
  name: string;
  type: string;
};

export type BooleanSchemaAttributeStats = CommonSchemaAttributeStats & {
  type: 'boolean';
  uniqueCount: number;
  categoryCounts?: Record<string, number>;
};

export type StringSchemaAttributeStats = CommonSchemaAttributeStats & {
  type: 'string';
  uniqueCount: number;
  categoryCounts?: Record<string, number>;
};

export type NumberSchemaAttributeStats = CommonSchemaAttributeStats & {
  type: 'number';
  minValue: number;
  maxValue: number;
  avgValue: number;
  stDevValue: number;
};

export type DateTimeSchemaAttributeStats = CommonSchemaAttributeStats & {
  type: 'datetime';
  firstDate: string;
  lastDate: string;
};

export type LocationSchemaAttributeStats = CommonSchemaAttributeStats & {
  type: 'location';
  minValue: string;
  maxValue: string;
};

/** Edge type, consist of a name, start point, end point and a list of attributes */
export type SchemaEdge = {
  name: string;
  to: string;
  from: string;
  collection: string;
  attributes: SchemaAttribute[];
  label: string;
  type?: string;
};

export type GraphAttributeDimensions = {
  nodes: { [id: string]: { [id: string]: string } };
  edges: { [id: string]: { [id: string]: string } };
};
