import { AttributeType } from '../statistics.types';

// Check if a string is a valid date in the YYYY-MM-DD format
const isValidDate = (value: string): boolean => {
  const dateRegex = /^\d{4}-\d{2}-\d{2}$/; // Matches YYYY-MM-DD format
  const [year, month, day] = value.split('-').map(Number);
  const date = new Date(value);

  // Check if the regex matches, and if the date is valid (correct month/day conversion)
  return (
    dateRegex.test(value) &&
    date.getFullYear() === year &&
    date.getMonth() + 1 === month && // Months are 0-based in JS Date
    date.getDate() === day
  );
};

// Check if a string is a valid time in the hh:mm:ss format
const isValidTime = (value: string): boolean => {
  const timeRegex = /^([01]\d|2[0-3]):([0-5]\d):([0-5]\d)$/;
  return timeRegex.test(value);
};

// Check if a string is a valid datetime in the YYYY-MM-DD hh:mm:ss format
const isValidDatetime = (value: string): boolean => {
  const datetimeRegex = /^\d{4}-\d{2}-\d{2} ([01]\d|2[0-3]):([0-5]\d):([0-5]\d)$/;
  const [date, time] = value.split(' ');

  return datetimeRegex.test(value) && isValidDate(date) && isValidTime(time);
};

// Check if a string is a valid number
const isValidNumber = (value: string): boolean => {
  return !isNaN(Number(value)) && !isNaN(parseFloat(value));
};

// Determines the type of an attribute
const getAttributeType = (value: any): AttributeType => {
  if (typeof value === 'string') {
    if (isValidNumber(value)) {
      return 'number';
    }
    if (isValidDatetime(value)) {
      return 'datetime';
    }
    if (isValidDate(value)) {
      return 'date';
    }
    if (isValidTime(value)) {
      return 'time';
    }
    return 'string';
  }

  if (typeof value === 'boolean') {
    return 'boolean';
  }

  if (typeof value === 'number') {
    return 'number';
  }

  if (Array.isArray(value)) {
    return 'array';
  }

  if (value instanceof Date) {
    return 'datetime';
  }

  if (typeof value === 'object' && value !== null) {
    return 'object';
  }

  return 'string';
};

export { getAttributeType };
